/******************************************************************************
               TER Constants
*******************************************************************************/
#if defined (_WIN32)
   #if !defined(WIN32)
     #define WIN32
   #endif
#endif
#if defined (WIN32)
   #define TER_CLASS      "Ter32Class"
#else
   #define TER_CLASS      "TerClass"
#endif

#define TER_DATA_VER       101

#define MAX_WIDTH          300

#define HILIGHT_OFF        0   
#define HILIGHT_LINE       1   
#define HILIGHT_CHAR       2   

#define WRAP_OFF           0        // suspend wrapping temporarily 
#define WRAP_MIN           1        // re-wrap minimum number of lines 
#define WRAP_PARA          2        // re-wrap all lines in the current paragraph 
#define WRAP_WIN           3        // re-wrap all lines visible in the current window 

#define APPLY_IGNORE       0        // ignore text data 
#define APPLY_MOD_CUR_LINE 1        // apply the text data to the current line 
#define APPLY_BEF_CUR_LINE 2        // insert after the current line 
#define APPLY_AFT_CUR_LINE 3        // insert after the current line 

#define SAVE_TER           0        // save in TER format
#define SAVE_TEXT          1        // save in text format
#define SAVE_TEXT_LINES    2        // text format with line breaks
#define SAVE_RTF           3        // save in Rich Text Format
#define SAVE_DEFAULT       4        // save in the format of the input file

#define ULINE              0x1   
#define BOLD               0x2   
#define ITALIC             0x4   
#define STRIKE             0x8   
#define SUPSCR             0x10  
#define SUBSCR             0x20  
#define HIDDEN             0x40  
#define PICT               0x80  
#define ULINED             0x100  
#define PROTECT            0x200  

#define DEFAULT_PFMT       0x0

#define CENTER             0x1
#define RIGHT_JUSTIFY      0x2
#define DOUBLE_SPACE       0x4
#define PARA_BOX_TOP       0x10
#define PARA_BOX_BOT       0x20
#define PARA_BOX_LEFT      0x40
#define PARA_BOX_RIGHT     0x80
#define PARA_BOX_DOUBLE    0x100
#define PARA_BOX_THICK     0x200
#define PARA_BOX_ATTRIB    0x3F0
#define LEFT               0x400
#define JUSTIFY            0x800
#define PAGE_HDR           0x1000
#define PAGE_FTR           0x2000
#define PAGE_HDR_FTR       0x3000

#define PARA_BOX           0xF0
                                                            
#define TAB_LEFT            0
#define TAB_RIGHT           1
#define TAB_CENTER          2
#define TAB_DECIMAL         3

#define TER_WORD_WRAP       0x1        // TER custom control styles
#define TER_PRINT_VIEW      0x2
#define TER_HSCROLL         0x4
#define TER_VSCROLL         0x8
#define TER_SHOW_STATUS     0x10
#define TER_SHOW_RULER      0x20
#define TER_BORDER_MARGIN   0x40
#define TER_OUTPUT_RTF      0x80
#define TER_USE_PARAMS      0x100
#define TER_READ_ONLY       0x200
#define TER_PAGE_MODE       0x400
#define TER_SHOW_TOOLBAR    0x800
#define TER_INVISIBLE       0x1000
#define TER_NUM_STYLES      13

#define SRCH_SCROLL         0x1
#define SRCH_SEARCH         0x2
#define SRCH_REPLACE        0x4
#define SRCH_RETRIEVE       0x8
#define SRCH_CASE           0x10
#define SRCH_WORD           0x20

#define ALIGN_BOT            0        // align picture bottom to base line
#define ALIGN_MIDDLE         1        // center picture vertically to baseline
#define ALIGN_TOP            2        // align picture top to base line

/******************************************************************************
    WIN32 specific defines
*******************************************************************************/
#if defined (WIN32)
   #define TER_HUGE
   #define _export
#else
   #define TER_HUGE huge 
#endif

/******************************************************************************
    More constant definitions
*******************************************************************************/
typedef int (FAR PASCAL _export *VBX_CALLBACK)(HWND, UINT,WPARAM,LPARAM); 

/****************************************************************************
   Instructions for packing the following structure
*****************************************************************************/
#ifndef RC_INVOKED
   #pragma pack(1)
#endif

/******************************************************************************
   Argument Structure:
   A set of control variables are passed by the user program to the 
   'CreateTerWindow' routine. The variables are passed using the structure 
   'arg_list',  as described below. The user program passes the POINTER to 
   this structure.
   This file must be included by your application module that calls the
   TER routine.  
*******************************************************************************/
struct arg_list { 
    int    x;         /* Initial x position of the editing window,
                         you may specify CW_USEDEFAULT to use default values.*/
    int    y;         // Initial y position of the editing window.
    int    width;     /* Initial width of the window in device units,
                         you may specify CW_USEDEFAULT to use default values.*/
    int    height;    /* Initial height of the editing window.  When you
                         use CW_USEDEFAULT for 'width', the 'height' is 
                         ingnored.*/
    long   LineLimit; /* Number of lines allowed in the editor window. 
                         Set to 0 to have unlimited number of lines*/ 

    BOOL   WordWrap;  /* Set this flag to true (1),  if you wish to enable the
                         word wrapping feature. */
    BOOL   PrintView; /* Set this flag to true if you wish the lines on
                         the screen to be wrapped current page margins.
                         Otherwise, the lines are wrapped at the end of
                         the window. This variable is ignored if 'WordWrap'
                         is set to FALSE */                      
    BOOL   PageMode;  /* Set this flag to true if you wish to edit one page
                         at a time */
    BOOL   ShowStatus;// show status line ?
    BOOL   ShowMenu;  // show the menu bar ? 
    BOOL   ShowHorBar;// show the horizonatal scroll bar 
    BOOL   ShowVerBar;// show the vertical scroll bar 
    BOOL   ruler;     // True to show the ruler
    BOOL   ToolBar;   // True to show the tool bar
    BOOL   UserCanClose;// TRUE allows the user to close the window 
    BOOL   BorderMargins;// display margins around the window borders 
    BOOL   ReadOnly;    // TRUE to create a read only session

    long   InitLine;  // Line number to position at the beginning. 
    BYTE   InputType; /* This flag specifies the input type. If you wish to
                         edit a file, set the input_type to 'F'. Conversely, 
                         if you wish to pass the text for editing in a buffer,
                         set this field to 'B'.*/

    BYTE   file[129];/*  If the input type is set to 'F',  specify the file
                         name for editing in this field. */

                     // ===== Buffer input fields only =======
    HGLOBAL hBuffer; /*  Specify the global memory handle containing the 
                         input text data.  This handle becomes the property
                         of the editor.  Your program must never try to lock
                         or free this handle */
    long BufferLen;  //  Buffer length in number of characters.                
    BYTE delim;      /*  Line delimiter character.  This character must be 
                         used to delimit individual lines within the 
                         text buffer.  You have an option of using a 
                         NULL, a carriage return or any other character of 
                         your choice. We recommend a NULL character as the
                         delimiter*/

    BYTE   padding1;  // This field is not used

    int    SaveFormat;// The format of the output file: SAVE_TER, SAVE_TEXT,
                      // SAVE_RTF and SAVE_AS_INPUT

    HINSTANCE hInst;  //  Handle of the current instanaces. 
    HWND   hParentWnd;// Handle to the parent window 
    HWND   hTextWnd; //  Ter main window handle, will be filled by TER later  
    DWORD  style;    //  Editor window style 
    BYTE   FontTypeFace[30]; // Default type face, example: TmwRmn Courier, etc 
    int    PointSize;// point size of the default font face 

    BOOL       open;     // TRUE indicates an open window
    int        modified; // For internal use 
    };


/******************************************************************************
   Field Data Structure:
   This structure is used to retrieve (GetTerFields()) and set (SetTerFields())
   control fields for a TER window.  
*******************************************************************************/
struct StrTerField { 
   /* The following fields are read/write fields.  To update a field, 
      you must retrieve the current values by calling the GetTerFields function.
      Modify the fields that you wish to, and then call the SetTerFields
      function to make the new value effective
   **************************************************************************/
      int      CurColW,           // current window column position                        
               PaintEnabledW,     // FALSE=disables the painting and word wrapping 
               WrapFlagW;         // wrap control, see WRAP_ constants 

      long     CurRowW,           // current window row position                           
               BeginLineW,        // line number of the first window line                  
               CurLineW;          // current line number in the file                       

      COLORREF TextBkColorW,      // background color for text 
               StatusBkColorW,    // background color of the status line 
               StatusColorW;      // foreground color of the status line 

      int      HilightTypeW,      // line or character block, see HILIGHT_ constants 
               HilightBegColW,    // beginning col 
               HilightEndColW;    // ending col 
               
      long     HilightBegRowW,    // beginning row 
               HilightEndRowW;    // ending row 

      BOOL     StretchHilightW;   // TRUE = highlight being stretch 


      BYTE     text[MAX_WIDTH];  // line text data, NULL terminated 
      BYTE     font[MAX_WIDTH];  // font id for every character in the 'text' array 

      int      pfmtW;             // paragraph id of the current line 
      int      LineLenW;          // length of the 'text' array 

      int      TextApply;        /* use this variable to specify how the 'text'
                                    and 'font' data should be applied to the
                                    current TER window, see APPLY_ constants */

      BOOL     ReclaimResourcesW;  // reuse unused font and para ids

      BOOL     ModifyProtectColorW,// modify the color or the protected text
               LinkDblClickW,      // TRUE to activate hyperlink on double click, otherwise use single click
               ShowProtectCaretW;  // TRUE to show caret on the protected text

      UINT     LinkStyleW;         // character style of the hyperlink phrase
      COLORREF LinkColorW;         // color of the linked phrase

   /* The following are the read only fields.  TER will ignore any 
      modification to these fields.
   **************************************************************************/
      HWND     hTerWndW;          // handle to the editor window 
      HDC      hTerDCW;           // handle to TER class DC 
      RECT     TerRectW,          // entire client window rectangal 
               TerWinRectW;       // text window rectangal 

      long     TotalLinesW,       // total lines in the file                               
               MouseLineW,        // text line position of mouse 
               MaxColBlockW;      // biggest column block allowed 

      int      TotalPfmtsW,       // total paragraph ids in use by the current window 
               TotalFontsW,       // Total font objects in use by the current window 
             
               WinWidthW,         // current window in columns                       
               WinHeightW,        // current window height in number of rows         
               TerWinOrgXW,       // window origin x co-ordinates 
               MouseColW;         // text column position of mouse 

      BOOL     modified;          // data modified, user needs to select the 'save' option to save data 
      BOOL     WordWrapW;         // TRUE when wordwrap turned on

      int      ParaLeftIndentW;   // paragraph left indent in twips
      int      ParaRightIndentW;  // paragraph right indent in twips
      int      ParaFirstIndentW;  // paragraph first line indent in twips
      UINT     ParaFlagsW;        // paragraph flags,includes such flags as double spacing,centering, right justify etc 
      int      ParaTabIdW;        // paragraph tab id, index into the tab table
      int      ParaCellIdW;       // paragraph cell id that the paragraph belongs to
      UINT     ParaShadingW;      // paragraph shading (0 to 10000)
      int      ParaFrameIdW;      // paragraph frame id
      int      ParaSpaceBeforeW;  // paragraph space before the paragraph in twips
      int      ParaSpaceAfterW;   // paragraph space after the paragraph in twips 
      int      ParaSpaceBetweenW; // paragraph space between the paragraph in twips 

      int      CurSectW;          // current section number

      int      LeftMarginW;       // section left margin in twips
      int      RightMarginW;      // section right margin in twips
      int      TopMarginW;        // section top margin in twips
      int      BotMarginW;        // section bottom margin in twips
      int      columnsW;          // number of columns in the current section

      int      CurPageW;          // current page number
      int      TotalPagesW;       // total pages

      int      MouseXW;           // last mouse x position
      int      MouseYW;           // last mouse y position
    };

/******************************************************************************
   Hyperlink Data Structure:
   This structure is used to pass hyperlink data to your application.
*******************************************************************************/
struct StrHyperLink {
      HWND    hWnd;               // Message windows
      BYTE    code[MAX_WIDTH+2];  // hyperlink code
      BYTE    text[MAX_WIDTH+2];  // hyperlink text
      BOOL    DoubleClick;        // TRUE if mouse double clicked, otherwise single click
    };

/******************************************************************************
   Print request parameter structure
   This structure is used to call the 'TerMergePrint' function to print
   a buffer or file containing text data in supported formats (RTF, native TER,
   and Ascii).
*******************************************************************************/
struct StrPrint { 
    BYTE   InputType; /* This flag specifies the input type. If you wish to
                         specify a disk file name, set the input_type to 'F'. 
                         Conversely, if you wish to pass the text for in a buffer,
                         set this field to 'B'.*/
    BYTE   file[129];/*  If the input type is set to 'F',  specify the input 
                         file name in this field. */

                     // ===== Buffer input fields only =======
    HANDLE  hBuffer; /*  Specify the global memory handle containing the 
                         input text data. */
    long BufferLen;  //  Buffer length in number of characters.                
    BYTE delim;      /*  Line delimiter character used to delimit the lines
                         in the input buffer.  This character can be a CR or
                         any other character. Not used for an RTF buffer */
    BYTE   padding1;  // This field is not used


    HDC    hDC;       // Printer or Window device context to print to.
                      // If NULL, the default printer
                      // device is opened.  The device context must be set to
                      // the pixel mapping mode.
    LPRECT rect;      // Printing area specified in mm (millimeter) units.
                      // If NULL, the entire page with default margins is used.
                      // The NULL value is not valid when printing to a 
                      // window device context
    long   StartPos;  // Starting character position.  Set to 0 to begin from
                      // the first page.
    BOOL   OnePage;   // Set to TRUE to print one (initial) page only.  The
                      // printed page is NOT spooled to the printer.
    long   NextPos;   // (OUTPUT) When OnePage is TRUE, the TerMergePrint function
                      // returns the character where the next page should begin.
                      // This value is set to 0 after the entire buffer is printed

                     // ===== Mail merge fields ==================
    LPBYTE  MergeFields;      // pointer to a list of mail merge field names.  
                             // The fields must be separated  by a '|' character. 
                             // The list must be terminated by a NULL character
    LPBYTE  MergeData;        // pointer to a list of mail merge data values.  
                             // Each data vlue must be separated  by a '|' character. 
                             // The list must be terminated by a NULL character
                             // The number of data elements in the MergeData
                             // MUST be same as the number of elements in the
                             // MergeFields array

                     // ===== Other fields ==================
    BOOL   PrintHiddenText;  // TRUE to print any hidden text
    HINSTANCE hInst;         // Instanaces handle of your program
    HWND   hParentWnd;       // A window handle in your application to receive mail merge messages

    int    NextY;            // (OUTPUT) The DLL returns the Y position on the paper
    };

/****************************************************************************
   Instructions to resume the default structure packing.
*****************************************************************************/
#ifndef RC_INVOKED
   #pragma pack()
#endif

/******************************************************************************
               TER Messages
*******************************************************************************/

/*******************************************************************************
   TER_CLOSE:
   Ter sends this message to the parent window (your application window)
   before closing.  The wParam variables contains the handle of the TER
   window.  lParam is not used.
   
   If your application is using a GlobalBuffer, you can retrieve the updated
   buffer when receiving this message. Use the GetTerBuffer() function to
   retrived the global handle to updated text data.
*******************************************************************************/
#define TER_CLOSE    (WM_USER)

/*******************************************************************************
   TER_MODIFIED:
   Ter sends this message to the parent window once when the editor text is 
   modified. The wParam variable contains the handle of the TER window.
*******************************************************************************/
#define TER_MODIFIED (WM_USER+1)

/*******************************************************************************
   TER_LINK:
   Ter sends this message to the parent window when the user clicks on
   the hyperlink text.
*******************************************************************************/
#define TER_LINK (WM_USER+2)

/*******************************************************************************
   TER_MERGE:
   Ter sends this message to the parent window to get the mail merge field
   data for a field name.  The TerMergePrint function first checks the
   supplied list of mail merge code.  Only if the required code is not
   found in the input list, it sends this message.
   The 'wParam' parameter contains the handle to the TER window, and
   the 'lParam' parameter contains the pointer to the field name.
   Your applicaton should return the pointer to the field text, or NULL if text not available.
*******************************************************************************/
#define TER_MERGE (WM_USER+3)

/*******************************************************************************
   TER_NOT_SAVED:
   Ter sends this message to the parent window (your application window)
   if the user abandons the modifications.  
   The wParam variables contains the handle of the TER window.  lParam is not used.
   The TER_CLOSE message always follows this message.
*******************************************************************************/
#define TER_NOT_SAVED    (WM_USER+4)


/******************************************************************************
                     Editor Function Prototype
   When calling TER from a C program, leave the TER_CPLUSPLUS symbol undefined.

   When calling TER from a C++ program, define the TER_CPLUSPLUS as following:

   #define TER_CPLUSPLUS extern "C"
   #include "ter.h"
*******************************************************************************/
#ifndef TER_CPLUSPLUS
   #if defined(__cplusplus) 
      #define TER_CPLUSPLUS extern "C"
   #else
      #define TER_CPLUSPLUS
   #endif
#endif

TER_CPLUSPLUS HWND   WINAPI _export CreateTerWindow(struct arg_list far *);
TER_CPLUSPLUS BOOL   WINAPI _export CloseTer(HWND,BOOL);
TER_CPLUSPLUS HGLOBAL WINAPI _export GetTerBuffer(HWND, long far *);
TER_CPLUSPLUS BOOL   WINAPI _export SetTerBuffer(HWND,HGLOBAL,long,LPBYTE,BOOL);
TER_CPLUSPLUS BOOL   WINAPI _export GetTerFields(HWND,struct StrTerField far *);
TER_CPLUSPLUS BOOL   WINAPI _export SetTerFields(HWND,struct StrTerField far *);
TER_CPLUSPLUS BOOL   WINAPI _export InsertRtfBuf(HWND,BYTE TER_HUGE *,long,long,int,BOOL);
TER_CPLUSPLUS BOOL   WINAPI _export SetTerFont(HWND,LPBYTE,BOOL);
TER_CPLUSPLUS BOOL   WINAPI _export SetTerDefaultFont(HWND,LPBYTE,int,UINT,COLORREF,BOOL);
TER_CPLUSPLUS BOOL   WINAPI _export SetTerPointSize(HWND,int,BOOL);
TER_CPLUSPLUS BOOL   WINAPI _export SetTerCharStyle(HWND,UINT,BOOL,BOOL);
TER_CPLUSPLUS BOOL   WINAPI _export SetTerColor(HWND,COLORREF,BOOL);
TER_CPLUSPLUS BOOL   WINAPI _export SetTerBkColor(HWND,COLORREF,BOOL);
TER_CPLUSPLUS BOOL   WINAPI _export InsertTerText(HWND,BYTE TER_HUGE *,BOOL);
TER_CPLUSPLUS BOOL   WINAPI _export SelectTerText(HWND,long,int,long,int,BOOL);
TER_CPLUSPLUS BOOL   WINAPI _export SetTerCursorPos(HWND,long,int,BOOL);
TER_CPLUSPLUS BOOL   WINAPI _export GetTerCursorPos(HWND,long far *,int far *);
TER_CPLUSPLUS BOOL   WINAPI _export SetTerParaFmt(HWND,UINT,BOOL,BOOL);
TER_CPLUSPLUS BOOL   WINAPI _export ParaIndentTwips(HWND,int,int,int,BOOL);
TER_CPLUSPLUS BOOL   WINAPI _export ParaLeftIndent(HWND,BOOL,BOOL);
TER_CPLUSPLUS BOOL   WINAPI _export ParaRightIndent(HWND,BOOL,BOOL);
TER_CPLUSPLUS BOOL   WINAPI _export ParaHangingIndent(HWND,BOOL,BOOL);
TER_CPLUSPLUS BOOL   WINAPI _export ParaNormal(HWND,BOOL);
TER_CPLUSPLUS BOOL   WINAPI _export DeselectTerText(HWND,BOOL);
TER_CPLUSPLUS BOOL   WINAPI _export SetTab(HWND,int,int,BOOL);
TER_CPLUSPLUS BOOL   WINAPI _export ClearTab(HWND,int,BOOL);
TER_CPLUSPLUS BOOL   WINAPI _export ClearAllTabs(HWND,BOOL);
TER_CPLUSPLUS BOOL   WINAPI _export TerMergeFields(HWND,LPBYTE,LPBYTE,BOOL); 
TER_CPLUSPLUS BOOL   WINAPI _export TerMergePrint(struct StrPrint far *);
TER_CPLUSPLUS BOOL   WINAPI _export TerMergePrintVB(struct StrPrint far *,LPRECT,LPBYTE,LPBYTE);
TER_CPLUSPLUS BOOL   WINAPI _export TerPageBreak(HWND,BOOL);
TER_CPLUSPLUS BOOL   WINAPI _export TerPrint(HWND,BOOL);
TER_CPLUSPLUS BOOL   WINAPI _export TerPrintPreviewMode(HWND,BOOL);
TER_CPLUSPLUS BOOL   WINAPI _export TerColBreak(HWND,BOOL);
TER_CPLUSPLUS BOOL   WINAPI _export TerEnableRefresh(HWND,BOOL);
TER_CPLUSPLUS BOOL   WINAPI _export TerLocateStyle(HWND,UINT,long far *,int far *,int far *);
TER_CPLUSPLUS BOOL   WINAPI _export TerSectBreak(HWND,BOOL);
TER_CPLUSPLUS BOOL   WINAPI _export TerSetModify(HWND,BOOL);
TER_CPLUSPLUS BOOL   WINAPI _export TerSetOutputFormat(HWND,int);
TER_CPLUSPLUS BOOL   WINAPI _export TerSetParaShading(HWND,int,BOOL);
TER_CPLUSPLUS BOOL   WINAPI _export LoadTerControl(void);
TER_CPLUSPLUS int    WINAPI _export GetTerLine(HWND,long,LPBYTE,LPBYTE);
TER_CPLUSPLUS BOOL   WINAPI _export SetTerLine(HWND,long,LPBYTE,LPBYTE);
TER_CPLUSPLUS BOOL   WINAPI _export GetFontInfo(HWND,int,LPBYTE,LPINT,UINT far *);
TER_CPLUSPLUS BOOL   WINAPI _export SaveTerFile(HWND,LPBYTE);
TER_CPLUSPLUS BOOL   WINAPI _export ReadTerFile(HWND,LPBYTE);
TER_CPLUSPLUS BOOL   WINAPI _export TerGetTextColor(HWND,int,LPDWORD,LPDWORD);
TER_CPLUSPLUS UINT   WINAPI _export TerMenuEnable(HWND,int);
TER_CPLUSPLUS UINT   WINAPI _export TerMenuSelect(HWND,int);
TER_CPLUSPLUS long   WINAPI _export TerRowColToAbs(HWND,long,int);
TER_CPLUSPLUS void   WINAPI _export TerAbsToRowCol(HWND,long,long far *,int far *); 
TER_CPLUSPLUS BOOL   WINAPI _export TerPrintPreview(HWND,HDC,RECT far *,int,BOOL);
TER_CPLUSPLUS BOOL   WINAPI _export TerSetMargin(HWND,int,int,int,int,BOOL);
TER_CPLUSPLUS BOOL   WINAPI _export TerSetReadOnly(HWND,BOOL);
TER_CPLUSPLUS BOOL   WINAPI _export TerEnableSpeedKey(HWND,int,BOOL);
TER_CPLUSPLUS BOOL   WINAPI _export TerRewrap(HWND);
TER_CPLUSPLUS BOOL   WINAPI _export TerSearchReplace(HWND,LPBYTE,LPBYTE,UINT,long,long far *,long far *);
TER_CPLUSPLUS int    WINAPI _export TerPastePicture(HWND,UINT,HGLOBAL,int,int,BOOL);
TER_CPLUSPLUS BOOL   WINAPI _export TerGetHypertext(HWND,LPBYTE,LPBYTE);
TER_CPLUSPLUS BOOL   WINAPI _export TerCreateTable(HWND,int,int,BOOL);
TER_CPLUSPLUS int    WINAPI _export TerPictureFromFile(HWND,LPBYTE,int,BOOL);
TER_CPLUSPLUS int    WINAPI _export TerInsertParaFrame(HWND,int,int,int,int);
TER_CPLUSPLUS BOOL   WINAPI _export TerMoveParaFrame(HWND,int,int,int,int,int);
TER_CPLUSPLUS BOOL   WINAPI _export GetTerFieldsAlt(HWND,struct StrTerField far *,long);
TER_CPLUSPLUS BOOL   WINAPI _export TerSetParaSpacing(HWND,int,int,int,BOOL);
TER_CPLUSPLUS int    WINAPI _export TerDataVersion(void);
TER_CPLUSPLUS BOOL   WINAPI _export TerAppendText(HWND,BYTE TER_HUGE *,int,int,BOOL);
TER_CPLUSPLUS BOOL   WINAPI _export TerInsertText(HWND,BYTE TER_HUGE *,int,int,BOOL);
TER_CPLUSPLUS int    WINAPI _export TerCreateFont(HWND,int,BOOL,LPBYTE,int,UINT,COLORREF,COLORREF,int,int);
TER_CPLUSPLUS int    WINAPI _export TerCreateParaId(HWND,int,BOOL,int,int,int,int,int,int,UINT,int,int,int,int,UINT);
TER_CPLUSPLUS BOOL   WINAPI _export TerSetAppMemory(HWND,long);
TER_CPLUSPLUS LPVOID WINAPI _export TerGetAppMemory(HWND);
TER_CPLUSPLUS int    WINAPI _export TerSetDefTabWidth(HWND,int,BOOL);
TER_CPLUSPLUS BOOL   WINAPI _export TerInsertObjectId(HWND,int,BOOL);
TER_CPLUSPLUS BOOL   WINAPI _export TerRegisterVbxCallback(HWND,VBX_CALLBACK);
TER_CPLUSPLUS BOOL   WINAPI _export TerGetPictInfo(HWND,int,LPINT,LPRECT,LPINT,LPINT);
TER_CPLUSPLUS BOOL   WINAPI _export TerSetPictInfo(HWND,int,UINT,int,int);

TER_CPLUSPLUS HANDLE WINAPI    _export StrToHandle(BYTE TER_HUGE *,long);        // needed for Visual Basic interface only  
TER_CPLUSPLUS BOOL   WINAPI    _export HandleToStr(BYTE TER_HUGE *,long,HANDLE); // needed for Visual Basic interface only  
TER_CPLUSPLUS LRESULT CALLBACK _export VbxProcessMessage(HWND,UINT,WPARAM,LPARAM,LPINT);

