/******************************************************************************
                Constant Definitions
*******************************************************************************/

#define MAX_TYPES          50    
#define MAX_POINTS         16    
#define MAX_FONTS          170    
#define MAX_SEGS           200
#define INITIAL_MAX_LINES  200
#define MAX_RTF_FONTS      200
#define MAX_RTF_COLORS     200
#define MAX_RTF_GROUPS     50
#define MAX_TABS           300
#define MAX_TAB_STOPS      20
#define MAX_SPACES         20
#define MAX_SECTS          100
#define MAX_FRAMES         1500
#define MAX_PARA_FRAMES    200
#define MAX_FIELDS         100
#define MAX_WIN_PTRS       200 
#define MAX_TASKS          50 
#define MAX_DRAG_OBJS      500
#define MAX_CELLS          800
#define MAX_TABLE_ROWS     400
#define MAX_SPEED_KEYS     200
#define MAX_PAGES          2500
#define MAX_PREVIEW_PAGES  2
#define MAX_ZOOMS          8

#define NUM_WIDTH_CHARS    256
#define BORDER_MARGIN      75

#define ID_FIRST_COMMAND   600

#define MIN_CELL_WIDTH     360    // min cell width in twips
#define WORD_ROW_INDENT    108    // standard indentation in MS WORD

#define SIGN_EOF16         0xBC
#define SIGN_EOF32         0xBA

#define DIB_TO_METAFILE  
                                                                                                                                    
#define PICT_DIBITMAP      0x0
#define PICT_METAFILE      0x1

#define OBJ_NONE           0x0
#define OBJ_EMBED          0x1
#define OBJ_LINK           0x2
#define OBJ_EMBED_TEMP     0x3
#define OBJ_EMBED_INSERT   0x4
                                                                                                                                    
#define DEFAULT_CFMT       0x0   
                                 
#define BOLD_WEIGHT        700   
#define NORM_WEIGHT        400   
                                 
#define UNIFORM            0xFFFF
                                 
                                 
#define MAX_PFMT           2000    
#define EACH_INDENT        360
#define MAX_INDENT         5760
#define MAX_PARA_PER_WIN   30
#define MAX_PARA_PER_PAGE  100

#define REDUCTION          2
#define CELL_MARGIN        180      // cell margin in twips
#define FRAME_MARGIN       75       // frame margin
#define FRAME_BORDER_WIDTH 15       // frame border width
                              
#define PAINT_MIN          1
#define PAINT_LINE         2
#define PAINT_WIN          3
#define PAINT_PARTIAL_WIN  5
#define PAINT_RANGE        6
#define PAINT_WIN_RESET    7

#define PAINT_DIRECT       1   
#define PAINT_MEM          2   
                               
#define HOR_SCROLL_MIN     0   
#define HOR_SCROLL_MAX     100 
#define VER_SCROLL_MIN     0   
#define VER_SCROLL_MAX     100 
                               
#define STATUS_ROW         -1 
#define MAX_LINES_PER_WIN  2000

#define RTF_BUF_LEN        4096
#define RTF_LINE_LEN       80

#define RTF_BEGIN          1
#define RTF_END            2
#define RTF_CONTROL        3
#define RTF_TEXT           4
#define RTF_PROPERTY       5
#define RTF_PICT           6
#define RTF_SECT           7
#define RTF_OBJECT         8

#define RTF_FILE_INCOMPLETE 1
#define RTF_SYNTAX_ERROR    2

#define RTF_FILE           0
#define RTF_BUF            1
#define RTF_CB             2
#define RTF_CB_BUF         3

#define PARAM_NONE          0
#define PARAM_INT           1
#define PARAM_DBL           2

#define TAB                 9
#define RULER_TOLERANCE     50
#define DEF_TAB_WIDTH       720      // default tab width in twips

#define PAGE_CHAR           12       // new page character
#define SECT_CHAR           20       // new section character
#define COL_CHAR            22       // new column character
#define ROW_CHAR            18       // new table row character
#define HDR_CHAR            17       // page header begin/end char
#define FTR_CHAR            16       // page footer begin/end char

#define MIN_LINES_PER_PAGE  20

#define BORDER_TOP          0x1
#define BORDER_BOT          0x2
#define BORDER_LEFT         0x4
#define BORDER_RIGHT        0x8
#define BORDER_ALL          0xF

#define BORDER_INDEX_TOP    0
#define BORDER_INDEX_BOT    1
#define BORDER_INDEX_LEFT   2
#define BORDER_INDEX_RIGHT  3

#define BORDER_PARA_TOP     0
#define BORDER_PARA_BOT     1
#define BORDER_PARA_LEFT    2
#define BORDER_PARA_RIGHT   3
#define BORDER_CELL_TOP     4
#define BORDER_CELL_BOT     5
#define BORDER_CELL_LEFT    6
#define BORDER_CELL_RIGHT   7
#define BORDER_ROW_TOP      8
#define BORDER_ROW_BOT      9
#define BORDER_ROW_LEFT    10
#define BORDER_ROW_RIGHT   11
#define MAX_BORDERS        12

#define BOX_TOP             0x10
#define BOX_BOT             0x20
#define BOX_LEFT            0x40
#define BOX_RIGHT           0x80
#define BOX_DOUBLE          0x100
#define BOX_THICK           0x200

#define INFO_TAB            0x1
#define INFO_SECT           0x2
#define INFO_PAGE           0x4
#define INFO_COL            0x8
#define INFO_CELL           0x10
#define INFO_ROW            0x20
#define INFO_TABLE          0x40
#define INFO_JUST           0x80
#define INFO_HDR            0x100
#define INFO_FTR            0x200
#define INFO_HDR_FTR        0x300
#define INFO_FRAME          0x400
#define INFO_SPACE_LINE     0x800
#define INFO_PAGE_NUMBER    0x1000

#define SECT_NEW_PAGE       0
#define SECT_CONTINUE       1

#define HYPER_CODE          HIDDEN

#define FIELD_NAME          ULINED

#define PROTECT_COLOR       0x4F4F4F       // color applied to protected text

#define BM_UNCHECKED        0  
#define BM_CHECKED          1  
#define BM_NONE             2  

#define MAX_DATA_LEN        100

#define MAX_PASTE_FORMATS   6
#define PASTE_OBJECT        0
#define PASTE_RTF           1
#define PASTE_TEXT          2
#define PASTE_PICT          3
#define PASTE_BM            4
#define PASTE_DIB           5

#define PASTE_DESC_OBJECT   "Object"
#define PASTE_DESC_RTF      "Formatted Text (RTF)"
#define PASTE_DESC_TEXT     "Unformatted Text"
#define PASTE_DESC_PICT     "Picture"
#define PASTE_DESC_BM       "Bitmap"
#define PASTE_DESC_DIB      "Device Independent Bitmap"

#define OLENOT_CHANGED      0x1
#define OLENOT_SAVED        0x2
#define OLENOT_CLOSED       0x4

#define MSDRAW_CLASS        "MSDraw"

#define FRAME_DIST_FROM_TEXT 180
#define FRAME_DIST_FROM_MARGIN 1440        // minimum distance from the margin

#define TER_IDLE            (WM_USER+10)   // message posted to do idle activity

// drag object types
#define DRAG_TYPE_SCR_FIRST    1
#define DRAG_TYPE_PICT         1
#define DRAG_TYPE_FRAME_SIZE   2
#define DRAG_TYPE_FRAME_MOVE   3
#define DRAG_TYPE_SCR_LAST     3

#define DRAG_TYPE_RULER_BEGIN  5
#define DRAG_TYPE_RULER_LEFT   5
#define DRAG_TYPE_RULER_RIGHT  6
#define DRAG_TYPE_RULER_FIRST  7
#define DRAG_TYPE_RULER_TAB    8
#define DRAG_TYPE_RULER_END    8

#define DRAG_TYPE_CELL         9
#define DRAG_TYPE_ROW_INDENT  10

#define DRAG_HOT_LEFT          0
#define DRAG_HOT_RIGHT         1
#define DRAG_HOT_TOP           2
#define DRAG_HOT_BOT           3

#define DRAG_TAB_WIDTH      100

#define CFLAG_SEL1             1      // cell selection flag
#define CFLAG_SEL2             2      // another cell selection flag

#define FRAME_RIGHTMOST      0x1      // rightmost frame
#define FRAME_LAST           0x2      // last text frame
#define FRAME_ON_SCREEN      0x4      // frame displayed on the screen

#define CURS_RESET           0        // used for cursor adjustment for hidden text
#define CURS_FORWARD         1
#define CURS_BACKWARD        2
#define CURS_UP              3
#define CURS_DOWN            4

#define PV_PAGE_DIST         1        // print preview page distance

#define PARA_FRAME_HPAGE     1        // align relative to the left edge of the page
#define PARA_FRAME_USED      2
#define PARA_FRAME_RIGHT_JUST 4
#define PARA_FRAME_CENTER    8

#define PARA_FRAME_MIN_WIDTH 360     // minimum width of a para frame

#define WIN_BOT_MARGIN       (ScrResY/4)

#define FIELD_ID_PAGE        1        // field id for the page field

#define LFLAG_PARA           0x1      // paragraph or cell break line
#define LFLAG_BREAK          0x2      // any other break line
#define LFLAG_PARA_FIRST     0x4      // first para line

// Short type defintions
#if defined (INC_OLE1)
   typedef int   (CALLBACK *CLIENT_CALLBACK)(LPOLECLIENT,OLE_NOTIFICATION,LPOLEOBJECT);
   typedef DWORD (CALLBACK *OLESTREAM_GET)(LPOLESTREAM,void FAR *,DWORD);
   typedef DWORD (CALLBACK *OLESTREAM_PUT)(LPOLESTREAM,OLE_CONST void FAR *,DWORD);
#else
   #define OLECLIPFORMAT WORD
   #if !defined(WIN32) 
      typedef LPSTR LPOLESTR;
   #endif
#endif

// unit conversion macros
#define    PointsToScrX(x)  (MulDiv(x,ScrResX,72))
#define    ScrToPointsX(x)  (MulDiv(x,72,ScrResX))

#define    PointsToScrY(x)  (MulDiv(x,ScrResY,72))
#define    ScrToPointsY(x)  (MulDiv(x,72,ScrResY))

#define    PointsToPrtX(x)  (MulDiv(x,PrtResX,72))
#define    PrtToPointsX(x)  (MulDiv(x,72,PrtResX))

#define    PointsToPrtY(x)  (MulDiv(x,PrtResY,72))
#define    PrtToPointsY(x)  (MulDiv(x,72,PrtResY))

#define    TwipsToScrX(x)  (MulDiv(x,ScrResX,1440))
#define    ScrToTwipsX(x)  (MulDiv(x,1440,ScrResX))

#define    TwipsToScrY(x)  (MulDiv(x,ScrResY,1440))
#define    ScrToTwipsY(x)  (MulDiv(x,1440,ScrResY))

#define    TwipsToPrtX(x)  (MulDiv(x,PrtResX,1440))
#define    PrtToTwipsX(x)  (MulDiv(x,1440,PrtResX))

#define    TwipsToPrtY(x)  (MulDiv(x,PrtResY,1440))
#define    PrtToTwipsY(x)  (MulDiv(x,1440,PrtResY))

#define    ScrToPrtX(x)    (MulDiv(x,PrtResX,ScrResX))
#define    PrtToScrX(x)    (MulDiv(x,ScrResX,PrtResX))

#define    ScrToPrtY(x)    (MulDiv(x,PrtResY,ScrResY))
#define    PrtToScrY(x)    (MulDiv(x,ScrResY,PrtResY))

#define    TwipsToInches(x)   (((float)(x))/1440)
#define    InchesToTwips(x)   ((long)((x)*1440))

#define    PointsToInches(x)   (((float)(x))/72)
#define    InchesToPoints(x)   ((long)((x)*72))

#define    TwipsToPoints(x)   ((x)/20)
#define    PointsToTwips(x)   ((x)*20)

#define    MmToInches(x)       (((float)(x))/(float)25.4)
#define    PrtToInchesX(x)     (((float)x)/PrtResX)
#define    PrtToInchesY(x)     (((float)x)/PrtResY)

#define    ResetUintFlag(flags,flag)  ((UINT)(flags)&(~((UINT)(flag))))
#define    ResetByteFlag(flags,flag)  ((BYTE)(flags)&(~((BYTE)(flag))))

/******************************************************************************
                         FAR/NEAR SELECTION
    Although TER is compiled in the medium model, in the Borland environment
    it is linked with the large model libraries.  The following variable
    defines the TERFAR constant accordingly.  This constant is used for
    modifying certain library functions.
*******************************************************************************/
#if defined(__TURBOC__)
   #ifdef __DLL__
      #define TERFAR far
      #define TERDLL
   #else
      #define TERFAR
   #endif
   #define DllMain DllEntryPoint
#else
   #ifdef _WINDLL
      #define TERDLL
   #endif
   #define TERFAR
   #define mktemp _mktemp
#endif

/******************************************************************************
    WIN32 specific defines
*******************************************************************************/
#define TERNT
#if defined (TER32)
   #undef  huge
   #define huge
   #undef  FreeProcInstance 
   #define FreeProcInstance(x) 
   #define GET_WM_COMMAND_ID(wp,lp) LOWORD(wp)
   #define CONTROL_ID(wp,lp)        LOWORD(wp)
   #define NOTIFY_MSG(wp,lp)        HIWORD(wp)
   #define SCROLL_MSG(wp,lp)        LOWORD(wp)
   #define THUMB_POS(wp,lp)         HIWORD(wp)
   #undef  farmalloc
   #define farmalloc  malloc
   #undef  farrealloc
   #define farrealloc realloc
   #undef  farfree
   #define farfree    free
   #define SIGN_EOF   SIGN_EOF32
#else
   #define GET_WM_COMMAND_ID(wp,lp) (wp)
   #define CONTROL_ID(wp,lp)        (wp)
   #define NOTIFY_MSG(wp,lp)        HIWORD(lp)
   #define SCROLL_MSG(wp,lp)        (wp)
   #define THUMB_POS(wp,lp)         LOWORD(lp)
   #define CCSTYLEFLAG DWORD
   #define SIGN_EOF    SIGN_EOF16
#endif


/******************************************************************************
     The following definitions are used to enhance the readability of the window
     specific global variables.  For example, instead of using ((w->TerArg),
     the program simply uses TerArg.
*******************************************************************************/

/******************************************************************************
     Text line data arrays
*******************************************************************************/
#define TerArg          (w->TerArgW)
#define hLine           (w->hLineW)      
#define LineLen         (w->LineLenW)
#define LineHt          (w->LineHtW)
#define LineY           (w->LineYW)
#define LineX           (w->LineXW)
#define LineFlags       (w->LineFlagsW)
#define hhLine          (w->hhLineW)
#define hLineLen        (w->hLineLenW)
#define hLineHt         (w->hLineHtW)
#define hLineY          (w->hLineYW)
#define hLineX          (w->hLineXW)
#define hLineFlags      (w->hLineFlagsW)

/******************************************************************************
     Screen display control variables
*******************************************************************************/
#define hTerWnd         (w->hTerWndW)
#define hTerDC          (w->hTerDCW)
#define hScrollBM       (w->hScrollBMW)
#define hPr             (w->hPrW)
#define hTerRegFont     (w->hTerRegFontW)
#define hTerCurFont     (w->hTerCurFontW)
#define hRulerFont      (w->hRulerFontW)
#define TerTextMet      (w->TerTextMetW)
#define hWaitCursor     (w->hWaitCursorW)
#define TerRect         (w->TerRectW)
#define TerWinRect      (w->TerWinRectW)
#define RulerRect       (w->RulerRectW)
#define hPagePen        (w->hPagePenW)
#define hColPen         (w->hColPenW)
#define hRowPen         (w->hRowPenW)
#define hCellPen        (w->hCellPenW)

#define CurRow          (w->CurRowW)
#define CurCol          (w->CurColW)
#define CurLine         (w->CurLineW)
#define CurLineY        (w->CurLineYW)
#define BeginLine       (w->BeginLineW)
#define TotalLines      (w->TotalLinesW)
#define MaxLines        (w->MaxLinesW)
#define PagingMargin    (w->PagingMarginW)
#define WinWidth        (w->WinWidthW)
#define WinHeight       (w->WinHeightW)
#define LineWidth       (w->LineWidthW)
#define PaintFlag       (w->PaintFlagW)
#define TabWidth        (w->TabWidthW)
#define HorScrollPos    (w->HorScrollPosW)
#define VerScrollPos    (w->VerScrollPosW)
#define TerWinWidth     (w->TerWinWidthW)
#define TerWinHeight    (w->TerWinHeightW)
#define CurPageWidth    (w->CurPageWidthW)
#define CurPageHeight   (w->CurPageHeightW)
#define CurTextHeight   (w->CurTextHeightW)
#define TerWinOrgX      (w->TerWinOrgXW)
#define TerWinOrgY      (w->TerWinOrgYW)
#define RulerClicked    (w->RulerClickedW)
#define FontsReleased   (w->FontsReleasedW)
#define WindowDestroyed (w->WindowDestroyedW)
#define ReclaimResources (w->ReclaimResourcesW)
#define NextX           (w->NextXW)
#define NextTab         (w->NextTabW)
#define NextSpace       (w->NextSpaceW)
#define RulerFontHeight (w->RulerFontHeightW)
#define ScrResX         (w->ScrResXW)
#define ScrResY         (w->ScrResYW)
#define PrtResX         (w->PrtResXW)
#define PrtResY         (w->PrtResYW)
#define PaintEnabled    (w->PaintEnabledW)
#define WrapPending     (w->WrapPendingW)
#define RulerPending    (w->RulerPendingW)
#define RulerSection    (w->RulerSectionW)
#define InAccelerator   (w->InAcceleratorW)
#define CursHorzPos     (w->CursHorzPosW)
#define OverhangFont    (w->OverhangFontW)
#define PaintBegLine    (w->PaintBegLineW)
#define PaintEndLine    (w->PaintEndLineW)
#define WinYOffset      (w->WinYOffsetW)
#define WinYOffsetLine  (w->WinYOffsetLineW)
#define CommandId       (w->CommandIdW)
#define RepaintNo       (w->RepaintNoW)

#define CrNewLine       (w->CrNewLineW)
#define CrSplitLine     (w->CrSplitLineW)
#define TabAlign        (w->TabAlignW)
#define JoinLines       (w->JoinLinesW)
#define Notified        (w->NotifiedW)

#define StatusRowPos    (w->StatusRowPosW)
#define StatusColPos    (w->StatusColPosW)
#define StatusLinePos   (w->StatusLinePosW)

#define CurClipRgn      (w->CurClipRgnW)

/******************************************************************************
     Text Frame Variables
*******************************************************************************/
#define hFrame          (w->hFrameW)
#define frame           (w->frameW)
#define CurFrame        (w->CurFrameW)
#define TotalFrames     (w->TotalFramesW)
#define MaxFrames       (w->MaxFramesW)
#define FirstTextFrame  (w->FirstTextFrameW)
#define LastTextFrame   (w->LastTextFrameW)

/******************************************************************************
                    Text to Screen mapping variables 
*******************************************************************************/
#define RowY            (w->RowYW)
#define RowHeight       (w->RowHeightW)
#define RowX            (w->RowXW)

/******************************************************************************
                       Text segment structure
*******************************************************************************/
#define TextSeg         (w->TextSegW)
#define hTextSeg        (w->hTextSegW)
#define TotalSegments   (w->TotalSegmentsW)
#define UseTextMap      (w->UseTextMapW)
#define TextBorder      (w->TextBorderW)
#define ParaShading     (w->ParaShadingW)
#define ParaFrameSpace  (w->ParaFrameSpaceW)

/******************************************************************************
     Mouse control variables
*******************************************************************************/
#define MouseCol        (w->MouseColW)
#define MouseLine       (w->MouseLineW)
#define MouseX          (w->MouseXW)
#define MouseY          (w->MouseYW)
#define MouseOverShoot  (w->MouseOverShootW)
#define IgnoreMouseMove    (w->IgnoreMouseMoveW)
#define VerySmallMovement  (w->VerySmallMovementW)

/******************************************************************************
     Word Wrapping related variables
*******************************************************************************/
#define DocCode           (w->DocCodeW)
#define DocBegin          (w->DocBeginW)
#define WrapMaxLines      (w->WrapMaxLinesW)
#define WrapBufferSize    (w->WrapBufferSizeW)
#define ParaChar          (w->ParaCharW)
#define DocExt            (w->DocExtW)
#define wrap              (w->wrapW)
#define hWrap             (w->hWrapW)
#define BufferLength      (w->BufferLengthW)
#define MaxBufferLength   (w->MaxBufferLengthW)
#define LastBufferedLine  (w->LastBufferedLineW)
#define LastWrappedLine   (w->LastWrappedLineW)
#define WrapFlag          (w->WrapFlagW)

/*****************************************************************************
                   Character Formatting Variables
******************************************************************************/
#define CfmtSign          (w->CfmtSignW)
#define cfmt              (w->cfmtW)
#define hCfmt             (w->hCfmtW)
#define WrapCfmt          (w->WrapCfmtW)
#define hWrapCfmt         (w->hWrapCfmtW)
#define TempCfmt          (w->TempCfmtW)
#define CurFmtId          (w->CurFmtIdW)
#define InputFontId       (w->InputFontIdW)

/******************************************************************************
                    Paragraph formatting variables
******************************************************************************/
#define hPfmt             (w->hPfmtW)
#define pfmt              (w->pfmtW)
#define hPfmtId           (w->hPfmtIdW)
#define PfmtId            (w->PfmtIdW)
#define TotalPfmts        (w->TotalPfmtsW)
#define CurPfmt           (w->CurPfmtW)
#define CurWrapPfmt       (w->CurWrapPfmtW)
#define tabw              (w->tabwW)
#define hTabw             (w->hTabwW)

/******************************************************************************
                       Font and Picture Control Data
******************************************************************************/
#define FmtSign           (w->FmtSignW)
#define FmtSignOld        (w->FmtSignOldW)
#define TerFont           (w->TerFontW)
#define hTerFont          (w->hTerFontW)
#define PrtFont           (w->PrtFontW)
#define hPrtFont          (w->hPrtFontW)
#define TerTab            (w->TerTabW)
#define hTerTab           (w->hTerTabW)
#define TerSect           (w->TerSectW)
#define hTerSect          (w->hTerSectW)
#define TerSect1          (w->TerSect1W)
#define hTerSect1         (w->hTerSect1W)
#define TotalFonts        (w->TotalFontsW)
#define UsingZoomFonts    (w->UsingZoomFontsW)
#define TotalTypes        (w->TotalTypesW)
#define TotalPointSizes   (w->TotalPointSizesW)
#define TotalTabs         (w->TotalTabsW)
#define DefTabWidth       (w->DefTabWidthW)
#define TotalSects        (w->TotalSectsW)
#define WrapSect          (w->WrapSectW)
#define SectModified      (w->SectModifiedW)
#define ReqTypeFace       (w->ReqTypeFaceW)
#define ReqPointSize      (w->ReqPointSizeW)

/******************************************************************************
     Print control variables
*******************************************************************************/
#define PageWidth         (w->PageWidthW)
#define PageHeight        (w->PageHeightW)
#define PageX             (w->PageXW)
#define AbortPrint        (w->AbortPrintW)
#define InPrinting        (w->InPrintingW)
#define InPrintPreview    (w->InPrintPreviewW)
#define PrinterAvailable  (w->PrinterAvailableW)
#define PrinterName       (w->PrinterNameW)
#define PrinterDriver     (w->PrinterDriverW)
#define PrinterPort       (w->PrinterPortW)
#define hDevMode          (w->hDevModeW)
#define pDevMode          (w->pDevModeW)
#define hAbortWnd         (w->hAbortWndW)
#define TotalPreviewPages (w->TotalPreviewPagesW)
#define PreviewZoom       (w->PreviewZoomW)
#define PreviewHeight     (w->PreviewHeightW)
#define PreviewWidth      (w->PreviewWidthW)
#define SavePvStatus      (w->SavePvStatusW)
#define SavePvRuler       (w->SavePvRulerW)
#define SavePvToolbar     (w->SavePvToolbarW)
#define SavePvWordWrap    (w->SavePvWordWrapW)
#define SavePvPrintView   (w->SavePvPrintViewW)
#define SavePvPageMode    (w->SavePvPageModeW)
#define SavePvShowHorBar  (w->SavePvShowHorBarW)
#define SavePvOrgX        (w->SavePvOrgXW)
#define SavePvOrgY        (w->SavePvOrgYW)
#define SavePvViewPageHdrFtr (w->SavePvViewPageHdrFtrW)
#define ZoomSubclassProc  (w->ZoomSubclassProcW)
#define OrigZoomSubclassProc (w->OrigZoomSubclassProcW)

/******************************************************************************
     Cursor adjustment variables
******************************************************************************/
#define PrevCursLine    (w->PrevCursLineW)
#define PrevCursCol     (w->PrevCursColW)
#define PrevCursPage    (w->PrevCursPageW)
#define PrevCursLineY   (w->PrevCursLineYW)
#define CursDirection   (w->CursDirectionW)

/******************************************************************************
     Color control variables
******************************************************************************/
#define TextDefBkColor    (w->TextDefBkColorW)
#define StatusBkColor     (w->StatusBkColorW)
#define StatusColor       (w->StatusColorW)
#define CurForeColor      (w->CurForeColorW)
#define CurBackColor      (w->CurBackColorW)
#define FrameShading      (w->FrameShadingW)
#define hTextBrush        (w->hTextBrushW)
#define hHilightBrush     (w->hHilightBrushW)
#define hStatusBrush      (w->hStatusBrushW)

/******************************************************************************
     Block highlighting variables
******************************************************************************/
#define HilightType       (w->HilightTypeW)
#define HilightBegRow     (w->HilightBegRowW)
#define HilightEndRow     (w->HilightEndRowW)
#define HilightBegCol     (w->HilightBegColW)
#define HilightEndCol     (w->HilightEndColW)
#define MaxColBlock       (w->MaxColBlockW)
#define StretchHilight    (w->StretchHilightW)

/******************************************************************************
     Tool Bar Variables
*******************************************************************************/
#define ToolBarHeight     (w->ToolBarHeightW)
#define hToolBarWnd       (w->hToolBarWndW)
#define hPvToolBarWnd     (w->hPvToolBarWndW)
#define lpToolBar         (w->lpToolBarW)
#define ToolBarCfmt       (w->ToolBarCfmtW)
#define ToolBarPfmt       (w->ToolBarPfmtW)
#define ToolBarFontProc   (w->ToolBarFontProcW)
#define OrigBarFontProc   (w->OrigBarFontProcW)
#define ToolBarPointProc  (w->ToolBarPointProcW)
#define OrigBarPointProc  (w->OrigBarPointProcW)

/******************************************************************************
     Search/Replace Variables
*******************************************************************************/
#define SearchString      (w->SearchStringW)
#define SearchDirection   (w->SearchDirectionW)
#define ReplaceString     (w->ReplaceStringW)
#define ReplaceWith       (w->ReplaceWithW)
#define SearchFlags       (w->SearchFlagsW)
#define ReplaceVerify     (w->ReplaceVerifyW)
#define ReplaceBlock      (w->ReplaceBlockW)

/******************************************************************************
     Undo variables
*******************************************************************************/
#define hUndo             (w->hUndoW)
#define hUndoCfmt         (w->hUndoCfmtW)
#define UndoBeg           (w->UndoBegW)
#define UndoEnd           (w->UndoEndW)
#define UndoExists        (w->UndoExistsW)
#define UndoType          (w->UndoTypeW)

/******************************************************************************
     Mail merge text variables
*******************************************************************************/
#define merge    (w->mergeW)

/******************************************************************************
     Hidden text variables
*******************************************************************************/
#define ShowHiddenText    (w->ShowHiddenTextW)

/******************************************************************************
     Paragraph Border and spacing variables
*******************************************************************************/
#define BoxLeft        (w->BoxLeftW)
#define BoxRight       (w->BoxRightW)
#define DlgOnFlags     (w->DlgOnFlagsW)
#define DlgOffFlags    (w->DlgOffFlagsW)
#define DlgLong        (w->DlgLongW)
#define DlgInt1        (w->DlgInt1W)
#define DlgInt2        (w->DlgInt2W)
#define DlgInt3        (w->DlgInt3W)
#define SpaceBef       (w->SpaceBefW)
#define SpaceAft       (w->SpaceAftW)

/******************************************************************************
     Paragraph Frame variables
*******************************************************************************/
#define ParaFrame       (w->ParaFrameW)
#define hParaFrame      (w->hParaFrameW)
#define CurParaFrame    (w->CurParaFrameW)
#define TotalParaFrames (w->TotalParaFramesW)
#define ContainsParaFrames (w->ContainsParaFramesW)

/******************************************************************************
     OLE Variables
*******************************************************************************/
#define TerClient      (w->TerClientW)
#define TerClientVtbl  (w->TerClientVtblW)
#define OleStream      (w->OleStreamW)
#define OleStreamVtbl  (w->OleStreamVtblW)
#define lhDoc          (w->lhDocW)
#define WaitForOle     (w->WaitForOleW)
#define OleReleaseMethod (w->OleReleaseMethodW)
#define hOleData       (w->hOleDataW)
#define pOleData       (w->pOleDataW)
#define CurObject      (w->CurObjectW)
#define pDocStorage    (w->pDocStorageW)

/******************************************************************************
     VBX Variables
*******************************************************************************/
#define VbxControl          (w->VbxControlW)
#define VbxMessageProcessed (w->VbxMessageProcessedW)
#define VbxCallback         (w->VbxCallbackW)

/******************************************************************************
     Drag object Variables
*******************************************************************************/
#define DragObj             (w->DragObjW)
#define hDragObj            (w->hDragObjW)
#define CurDragObj          (w->CurDragObjW)
#define CurHotSpot          (w->CurHotSpotW)
#define TotalDragObjs       (w->TotalDragObjsW)
#define PictureHilighted    (w->PictureHilightedW)
#define FrameTabsHilighted  (w->FrameTabsHilightedW)
#define FrameRectHilighted  (w->FrameRectHilightedW)
#define PictureClicked      (w->PictureClickedW)
#define FrameClicked        (w->FrameClickedW)
#define hFocusPen           (w->hFocusPenW)

/******************************************************************************
     Table Variables
*******************************************************************************/
#define TableRow             (w->TableRowW)
#define cell                 (w->cellW)
#define hTableRow            (w->hTableRowW)
#define hCell                (w->hCellW)
#define TotalTableRows       (w->TotalTableRowsW)
#define TotalCells           (w->TotalCellsW)
#define TableRows            (w->TableRowsW)
#define TableCols            (w->TableColsW)
#define CellChar             (w->CellCharW)
#define FrameCellId          (w->FrameCellIdW)
#define FrameRowId           (w->FrameRowIdW)
#define FrameHeight          (w->FrameHeightW)
#define ShowTableGridLines   (w->ShowTableGridLinesW)

/******************************************************************************
     RTF Control Variables
******************************************************************************/
#define RtfCurRowId          (w->RtfCurRowIdW)
#define RtfCurCellId         (w->RtfCurCellIdW)
#define RtfParaFID           (w->RtfParaFIDW)
#define RtfParaFrameInfo     (w->RtfParaFrameInfoW)
#define RtfLastCellId        (w->RtfLastCellIdW)
#define RtfInTable           (w->RtfInTableW)
#define RtfInHdrFtr          (w->RtfInHdrFtrW)

/******************************************************************************
     Page Header/Footer Variables
******************************************************************************/
#define ViewPageHdrFtr       (w->ViewPageHdrFtrW)
#define EditPageHdrFtr       (w->EditPageHdrFtrW)
#define PosPageHdrFtr        (w->PosPageHdrFtrW)
#define HdrFirstPageLine     (w->HdrFirstPageLineW)
#define HdrLastPageLine      (w->HdrLastPageLineW)
#define FtrFirstPageLine     (w->FtrFirstPageLineW)
#define FtrLastPageLine      (w->FtrLastPageLineW)

/******************************************************************************
     Caret Variables
*******************************************************************************/
#define CaretHeight      (w->CaretHeightW)
#define CaretEnabled     (w->CaretEnabledW)
#define CaretHidden      (w->CaretHiddenW)
#define CaretPositioned  (w->CaretPositionedW)
#define CaretEngaged     (w->CaretEngagedW)
#define CaretPos         (w->CaretPosW)
#define CaretPage        (w->CaretPageW)
#define ShowProtectCaret (w->ShowProtectCaretW)

/******************************************************************************
     Other Variables
*******************************************************************************/
#define BreakChars        (w->BreakCharsW)
#define FileInfo          (w->FileInfoW)
#define InsertMode        (w->InsertModeW)
#define TerHelpWanted     (w->TerHelpWantedW)
#define WindowBeingCreated (w->WindowBeingCreatedW)
#define CrLfUsed          (w->CrLfUsedW)
#define ProtectionLock    (w->ProtectionLockW)
#define ShowParaMark      (w->ShowParaMarkW)
                                                    
#define TerHelpFile       (w->TerHelpFileW)
#define hTerAccTable      (w->hTerAccTableW)

#define RtfClipFormat        (w->RtfClipFormatW)
#define NativeClipFormat     (w->NativeClipFormatW)
#define ObjectLinkClipFormat (w->ObjectLinkClipFormatW)
#define OwnerLinkClipFormat  (w->OwnerLinkClipFormatW)

#define FileFormat        (w->FileFormatW)
#define RtfInput          (w->RtfInputW)

#define TotalPages        (w->TotalPagesW)
#define MaxPages          (w->MaxPagesW)
#define PageModifyCount   (w->PageModifyCountW)
#define CurPage           (w->CurPageW)
#define PageLine          (w->PageLineW)
#define hPageLine         (w->hPageLineW)
#define RepageBeginLine   (w->RepageBeginLineW)
#define repaginating      (w->repaginatingW)
#define SpeedKeyEnabled   (w->SpeedKeyEnabledW)
#define ShowHyperlinkCursor (w->ShowHyperlinkCursorW)
#define MatchIds          (w->MatchIdsW)
#define hAppMemory        (w->hAppMemoryW)
#define pAppMemory        (w->pAppMemoryW)
#define ModifyProtectColor (w->ModifyProtectColorW)
#define LinkStyle         (w->LinkStyleW)
#define LinkColor         (w->LinkColorW)
#define LinkDblClick      (w->LinkDblClickW)

